#include "gui.h"
#include "qfont.h"
#include "kapp.h"
#include "qlayout.h"

#include "hostlist.h"
#include "hconfig.h"

#include "guiinterface.h"
#include "gui.moc"

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>

#define _(A) (klocale->translate(A))

char global_testonly;

ChooserGUI *chooserGUI;


class MyApp:public KApplication {
public:
  MyApp( int &argc, char **argv );
  virtual bool x11EventFilter( XEvent * );
};

MyApp::MyApp(int &argc, char **argv ):
  KApplication(argc, argv)
{}

bool MyApp::x11EventFilter( XEvent * ev)
{
  if( ev->type == KeyPress){
    if ( XLookupKeysym(&(ev->xkey),0) == XK_Return)
      chooserGUI->slotAccept();
  }
  return FALSE;
}



ChooserGUI::ChooserGUI(KHostConfig *conf, QWidget *parent, const char *name):
  QDialog( parent, name, false, 
	  WStyle_Customize | WStyle_NoBorder | WStyle_Tool)
{

  QBoxLayout *toplevel_layout, *lines;
  QHBoxLayout *line_1, *line_2;
  QSize size;
  int w, h;

  QFrame* winFrame = new QFrame( this);
  winFrame->setFrameStyle(QFrame::WinPanel| QFrame::Raised);


  toplevel_layout = new QHBoxLayout(winFrame);

  lines = new QVBoxLayout();
  line_1= new QHBoxLayout();
  line_2= new QHBoxLayout();

  toplevel_layout->addSpacing(10);
  toplevel_layout->addLayout(lines,1);
  toplevel_layout->addSpacing(10);

  lines->addSpacing(10);
  lines->addLayout(line_1);

  if (!conf->getHeaderLogo()->isNull()) {
    logo = new QLabel(winFrame);
    logo->setPixmap(*conf->getHeaderLogo());
    logo->setMinimumSize(logo->sizeHint());
    logo->setMaximumSize(logo->sizeHint());
  } else
    logo = 0;
  header = new QLabel(winFrame);
  header->setFont(conf->getHeaderFont());
  header->setText(conf->getHeaderText());
  header->setMinimumSize(header->sizeHint());
  header->setMaximumSize(header->sizeHint());

  if (logo)
    line_1->addWidget(logo);
  line_1->addStretch(1);
  line_1->addWidget(header);
  line_1->addStretch(1);

  hostlist = new QListBox(winFrame);
  hostlist-> setFont(conf->getHostListFont());

  lines->addSpacing(10);
  lines->addWidget(hostlist,1);
  lines->addSpacing(10);

  if (strlen(conf->getMessageText()) != 0) {
    // the optional message text
    msg = new QLabel(winFrame);
    msg->setFont(conf->getMessageFont());
    msg->setText(conf->getMessageText());
    msg->setMinimumSize(msg->sizeHint());
    msg->setMaximumSize(msg->sizeHint());

    lines->addWidget(msg, 0, AlignLeft);
    lines->addSpacing(10);
  } else 
    msg = 0;

  lines->addLayout(line_2);
  lines->addSpacing(10);

  cancel = new QPushButton(winFrame);
  cancel->setFont(conf->getButtonFont());
  cancel->setText(_("&Cancel"));
  size = cancel->sizeHint();
  cancel->setMinimumHeight(size.height());
  cancel->setMaximumHeight(size.height());


  accept = new QPushButton(winFrame);
  accept->setFont(conf->getButtonFont());
  accept->setText(_("&Accept"));
  accept->setMinimumHeight(size.height());
  accept->setMaximumHeight(size.height());
  ping   = new QPushButton(winFrame);
  ping->setFont(conf->getButtonFont());
  ping->setText(_("&Ping"));
  ping->setMinimumHeight(size.height());
  ping->setMaximumHeight(size.height());
  
  line_2->addWidget(cancel,1);
  line_2->addSpacing(10);
  line_2->addWidget(accept,1);
  line_2->addSpacing(10);
  line_2->addWidget(ping,1);

  toplevel_layout->activate();
  accept->setEnabled(FALSE);
  current_item = 0;
  rec_packet = 0;
  ping_timer = new QTimer();


  connect(cancel, SIGNAL(clicked()), SLOT(slotCancel()));
  if (!global_testonly) {
    connect(accept, SIGNAL(clicked()), SLOT(slotAccept()));
    connect(ping, SIGNAL(clicked()), SLOT(slotPing()));
    connect(hostlist, SIGNAL(highlighted(const char *)), SLOT(slotHighlighted(const char *)));
    connect(ping_timer, SIGNAL(timeout()), SLOT(slotPingTimeout()));
    connect(hostlist, SIGNAL(selected(int)), SLOT(slotListDblClick(int)));
  }
  w = conf->getWidth();
  h = conf->getHeight();
  winFrame->setMinimumSize(w, h);
  setGeometry( (QApplication::desktop()->width()-w)/2,
	       (QApplication::desktop()->height()-h)/2,
	       w,h);
  namewidth = conf->getNameWidth();
}




void ChooserGUI::setHostList(char ** list, int listlen)
{
  KHostEntry *entry;
  int i, w;

  hostlist->clear();
  if (listlen > 0) {
    w = hostlist->width();
    for(i=0; i < listlen; i++) {
      entry = new KHostEntry(list[i], (namewidth*w)/100);
      hostlist->insertItem(entry);
    }
  }

  i = hostlist->currentItem();
  if (i != -1) {
    // CC: The user has already selected a host, so fake a
    // highliht event
    slotHighlighted(hostlist->text(i));
  } else {
    accept->setEnabled(0);
    if (0 != current_item)
      delete current_item;
    current_item = 0;
  }
}


void ChooserGUI::initReceiverSocket(int socknum)
{
  if (0 != rec_packet) {
    disconnect(SLOT(slotReceivePacket(int)));
    delete(rec_packet);
  }
  rec_packet = new QSocketNotifier(socknum, QSocketNotifier::Read);
  connect(rec_packet, SIGNAL(activated(int)), SLOT(slotReceivePacket(int)));
  rec_packet->setEnabled(TRUE);
}



void ChooserGUI::startPingTimer(int timeout)
{
  ping_timer->start(timeout, TRUE);
}


void ChooserGUI::slotHighlighted(const char *item)
{
  if (DoCheckWillingQt(item)) {
    accept->setEnabled(TRUE);
    if (0 != current_item)
      delete current_item;
    current_item = qstrdup(item);
  } else {
    accept->setEnabled(FALSE);
    if (0 != current_item)
      delete(current_item);
    current_item = 0;
  }
}



void ChooserGUI::slotCancel()
{
  DoCancelQt();
}



void ChooserGUI::slotAccept()
{
  if (0 == current_item) {
    QApplication::beep();
  } else {
    DoAcceptQt(current_item);
  }
}



void ChooserGUI::slotPing()
{
  DoPingQt();
}


void ChooserGUI::slotPingTimeout()
{
  PingHostsQt();
}



void ChooserGUI::slotListDblClick(int)
{
  slotAccept();
}


void ChooserGUI::slotReceivePacket(int )
{
  ReceivePacketQt();
}


void SetHostList(char ** list, int listlen)
{
  chooserGUI->setHostList(list, listlen);
}

void MonitorSocketRead(int socknum)
{
  chooserGUI->initReceiverSocket(socknum);
}


void PingAgain(int timeout)
{
  chooserGUI->startPingTimer(timeout);
}

int retval;

void DoExit(int value)
{
  delete chooserGUI;
  if(!global_testonly)
    QApplication::desktop()->setActiveWindow();
  exit(value);
}


char *list[] = {
	"klapauzius.ceci.at Willing to Manage",
	"tichy.ceci.at        Willing to Manage",
	"trurl.ceci.at    Willing to Manage"
};

int main(int argc, char *argv[])
{
 MyApp app(argc, argv);
 KHostConfig *conf;
 QWidget *w;

 int i;

 global_testonly = 0;
 for(i=1; i < argc; i++) {
   if(!strcmp(argv[i], "-test"))
     global_testonly = 1;

   if(!strcmp(argv[i], "-version")) {
     printf("KDE Host Chooser Version %s\n", KHOSTCHOOSER_VERSION);
     exit(0);
   }

   if(!strcmp(argv[i], "-help")) {
     printf("KDE Host Chooser Version %s\n", KHOSTCHOOSER_VERSION);
     printf("Usage: khostchooser [-test] [-version] [-help]\n");
     exit(0);
   }
 }

 conf = new KHostConfig();

 QApplication::setStyle(conf->getGUIStyle());

 if(global_testonly) {
   w = new QWidget();
   chooserGUI = new ChooserGUI(conf, w);
   
   chooserGUI->move(50,50);
   
   if (conf->getBgColor() != 0)
     w->setBackgroundColor(*conf->getBgColor());
   chooserGUI->setHostList(list, 3);  
   w->show();
   w->resize(chooserGUI->width()+100, chooserGUI->height()+100);
   app.setMainWidget(w);
 } else {
   if (conf->getBgColor() != 0)
     QApplication::desktop()->setBackgroundColor(*conf->getBgColor());
   chooserGUI = new ChooserGUI(conf);
   InitXDMCP(argc, argv);
   chooserGUI->setActiveWindow();
   app.setMainWidget(chooserGUI);
 }
 chooserGUI->show();

 app.exec();
 return 0;
}


